/*
 * Copyright (c) 2005-2010 KOM – Multimedia Communications Lab
 *
 * This file is part of PeerfactSim.KOM.
 * 
 * PeerfactSim.KOM is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 * 
 * PeerfactSim.KOM is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with PeerfactSim.KOM.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

package de.tud.kom.p2psim.api.topology.movement;

import java.util.LinkedList;
import de.tud.kom.p2psim.impl.energy.components.ActuatorComponent;
import de.tud.kom.p2psim.impl.topology.util.PositionVector;
import de.tudarmstadt.maki.simonstrator.api.uavsupport.callbacks.ReachedLocationCallback;

/**
 * UAV Movement Models provide basic functionality for the simulation of flying UAVs. 
 * 
 * @author Julian Zobel
 * @version 1.0, 20.01.2020
 */
public interface UAVMovementModel {
	
	// NO - NO CHANGY
	public final double AIRDENSITY = 1.2045; // kg/m^3
	public final double GRAVITY = 9.807; // m/s^2
	
	public void setMotorControl(ActuatorComponent motor);
	
	/**
	 * Set the preferred/target speed of this UAV.
	 * 
	 * @param targetVelocity
	 */
	public void setTargetVelocity(double targetVelocity);
	
	public double getVerticalAscentMaxVelocity();
	public double getHorizontalMaxVelocity();
	public double getCurrentVelocity();
	public double getHorizontalMinVelocity();
	
	/**
	 * Estimate the power consumption for a given velocity
	 * @param velocity 
	 * @return The power consumption in W for the given velocity.
	 */
	public double estimatePowerConsumptionWatt(double velocity);
	public double estimateOptimalSpeed();
	
	public void move(long timeBetweenMovementOperations);
	
	public void setTargetLocation(PositionVector target, ReachedLocationCallback reachedLocationCallback);
	public void setTargetLocationRoute(LinkedList<PositionVector> route, ReachedLocationCallback reachedLocationCallback);
	public void addTargetLocation(PositionVector target, ReachedLocationCallback reachedLocationCallback);
	
	/**
	 * Returns a copy of the current target locations
	 * 
	 * @return
	 */
	public LinkedList<PositionVector> getTargetLocations();
	
	public void removeTargetLocations();
}
