/*
 * Copyright (c) 2005-2010 KOM – Multimedia Communications Lab
 *
 * This file is part of PeerfactSim.KOM.
 * 
 * PeerfactSim.KOM is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 * 
 * PeerfactSim.KOM is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with PeerfactSim.KOM.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

package de.tud.kom.p2psim.impl.energy;

import de.tud.kom.p2psim.api.energy.Battery;
import de.tudarmstadt.maki.simonstrator.api.util.XMLConfigurableConstructor;

/**
 * Extension to the basic battery implementation, enables to charge the battery.
 * 
 * 
 * @author Julian Zobel
 * @version 1.0, 21.09.2018
 */
public class RechargeableBattery extends SimpleBattery {

	@XMLConfigurableConstructor({ "capacity", "initialEnergy" })
	public RechargeableBattery(double capacity, double initialEnergy) {
		super(capacity, initialEnergy);		
	}
	
	/**
	 * Charge the battery with the given amount in <b>uJ</b>
	 * 
	 * @param charge
	 */
	public void chargeBattery(double charge) {
		if(currentEnergy + charge >= capacity) {
			currentEnergy = capacity;
		}
		else {
			currentEnergy += charge;
		}
	}	
	
	@Override
	public void setToMaximumLoad() {
		currentEnergy = capacity;		
	}
	
	@Override
	public Battery copy(double initialEnergy) {
		return new RechargeableBattery(capacity  / 1000000, initialEnergy  / 1000000);
	}

	@Override
	public String toString() {
		return "Rechargeable Battery ["+getCurrentPercentage()+"%] ["+getCurrentEnergy()+"/"+getMaximumEnergy()+"]";
	}
}
