/*
 * Copyright (c) 2005-2010 KOM – Multimedia Communications Lab
 *
 * This file is part of PeerfactSim.KOM.
 * 
 * PeerfactSim.KOM is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 * 
 * PeerfactSim.KOM is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with PeerfactSim.KOM.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

package de.tud.kom.p2psim.impl.topology.movement.modularosm.attraction;

import java.util.LinkedList;
import java.util.List;
import de.tud.kom.p2psim.impl.topology.util.PositionVector;
import de.tudarmstadt.maki.simonstrator.api.util.XMLConfigurableConstructor;

/**
 * Attraction Generator, providing 4 equidistant attraction points, i.e., in a square
 * 
 * @author  Julian Zobel
 * @version 1.0, Nov 2018
 */
public class EquidistantSquareAttractionGenerator extends AbstractAttractionProvider {

	private PositionVector worldDimension;
	private double squareSize;
	
	@XMLConfigurableConstructor({ "squareSize" })
	public EquidistantSquareAttractionGenerator(double squareSize) {
		this.squareSize = squareSize;
		createAttractionPoints();
	}

	@Override
	public LinkedList<AttractionPoint> getAttractionPoints() {
		if(super.getAttractionPoints().isEmpty())
			createAttractionPoints();
		
		return super.getAttractionPoints();
	}
	
	private void createAttractionPoints() {
						
		double x = worldDimension.getX() / 2;
		double y = worldDimension.getY() / 2;
		
		PositionVector p1 = new PositionVector(x - squareSize, y - squareSize);	
		this.addAttractionPoint(new AttractionPoint("AP1", p1));
		
		PositionVector p2 = new PositionVector(x - squareSize, y + squareSize);		
		this.addAttractionPoint(new AttractionPoint("AP2", p2));		
		
		PositionVector p3 = new PositionVector(x + squareSize, y - squareSize);		
		this.addAttractionPoint(new AttractionPoint("AP3", p3));		
		
		PositionVector p4 = new PositionVector(x + squareSize, y + squareSize);		
		this.addAttractionPoint(new AttractionPoint("AP4", p4));	
	}

	
}
