/*
 * Copyright (c) 2005-2010 KOM – Multimedia Communications Lab
 *
 * This file is part of PeerfactSim.KOM.
 * 
 * PeerfactSim.KOM is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 * 
 * PeerfactSim.KOM is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with PeerfactSim.KOM.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

package de.tud.kom.p2psim.impl.topology.movement.modularosm.transition;

import de.tud.kom.p2psim.api.topology.movement.SimLocationActuator;
import de.tud.kom.p2psim.impl.topology.movement.modularosm.attraction.IAttractionProvider;
import de.tudarmstadt.maki.simonstrator.api.component.sensor.location.IAttractionPoint;

/**
 * This is the interface for the Transition Strategy.<br>
 * 
 * @author Martin Hellwig
 * @version 1.0, 03.07.2015
 * 
 * 
 * 
 * @author Julian Zobel
 * @version 1.1, 27.01.2020
 */
public interface IAttractionAssigmentStrategy {

	/**
	 * Currently assigned attraction point.
	 * 
	 * @param comp
	 * @return
	 */
	public IAttractionPoint getAssignment(SimLocationActuator host);
	
	public IAttractionPoint getLastAssignment(SimLocationActuator host);
	
	public void addAttractionAssignmentListener(AttractionAssignmentListener listener);
	
	public void removeAttractionAssignmentListener(AttractionAssignmentListener listener);

	public void setAttractionProvider(IAttractionProvider attractionProvider);
	
	/**
	 * Add the object and assign the MS to an {@link IAttractionPoint}.
	 * 
	 * @param ms
	 */
	public void addComponent(SimLocationActuator host);

	/**
	 * Notify the TransitionStrategy, that the component has reached an
	 * attraction point.
	 * 
	 * @param ms
	 */
	public void reachedAttractionPoint(SimLocationActuator host, IAttractionPoint attractionPoint);

	/**
	 * Updates the target attraction point of a component
	 * 
	 * @param attractionPoint
	 */
	public void updateTargetAttractionPoint(SimLocationActuator comp,
			IAttractionPoint attractionPoint);

	public interface AttractionAssignmentListener {

		/**
		 * Invoked after the assignment of AttractionPoints for the given
		 * component changed.
		 * 
		 * @param component
		 * @param newAssignment
		 */
		public void updatedAttractionAssignment(SimLocationActuator host,
				IAttractionPoint newAssignment);

	}
		
	public boolean hostInAttractionPointArea(SimLocationActuator host);
	
	public long getPauseTime(IAttractionPoint attractionPoint);
}
