/*
 * Copyright (c) 2005-2010 KOM – Multimedia Communications Lab
 *
 * This file is part of PeerfactSim.KOM.
 * 
 * PeerfactSim.KOM is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 * 
 * PeerfactSim.KOM is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with PeerfactSim.KOM.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

package de.tud.kom.p2psim.impl.topology.waypoints;

import java.util.Random;

import de.tud.kom.p2psim.api.topology.Topology;
import de.tud.kom.p2psim.impl.topology.DefaultTopology;
import de.tud.kom.p2psim.impl.topology.util.PositionVector;
import de.tudarmstadt.maki.simonstrator.api.Binder;
import de.tudarmstadt.maki.simonstrator.api.Randoms;

public class RandomWaypointGenerator {
	
	private Random rand;
	private PositionVector worldDimension;
	
	public RandomWaypointGenerator() {
		super();
		
		this.worldDimension = Binder.getComponentOrNull(Topology.class)
				.getWorldDimensions();
		
		this.rand = Randoms.getRandom(RandomWaypointGenerator.class);
	}
	
	public PositionVector getRandomPoint() {
		double x = rand.nextDouble() * Math.nextUp(worldDimension.getX());
		double y = rand.nextDouble() * Math.nextUp(worldDimension.getY());
		
		PositionVector pos = new PositionVector(x, y);
		
		if(!DefaultTopology.isWithinWorldBoundaries(pos)) {
			throw new UnsupportedOperationException("[RandomWaypointGenerator] The random generated point must be inside the simulation area!");
		}
		
		return pos;
	}
}